#!/usr/bin/env bash

requests=("invite ack bye cancel register options prack subscribe notify publish info refer message update")
for request in "${requests[@]}"
do
	python test.py -r "${request}"
done
