$requests = @(
	"invite",
	"ack",
	"bye",
	"cancel",
	"register",
	"options",
	"prack",
	"subscribe",
	"notify",
	"publish",
	"info",
	"refer",
	"message",
	"update"
)

foreach ($request in $requests)
{
    #Write-Output ("Trying " + $request.ToUpper() + "...")
    & python test.py -r $request
}
